﻿#pragma strict

var fxCoolDown : float = 0.75;
var BloodPerHit : float = 2.5;
var Punch : float = 0.1;
var Pain : AudioClip[];
var Armor : float = 50;
var Regen : float = 0.1;
var Destruction : float = 0.5;
var Vulnerability : int = 0;

private var Size : Vector3;

function Start () {
	Size = gameObject.transform.localScale;
}

function Update () {
	gameObject.transform.localScale =  Vector3.Lerp(gameObject.transform.localScale, Size, Regen * Time.deltaTime);

}


private var fxDeadline : float = 0;
private var collisionEvents = new ParticleSystem.CollisionEvent[16];

function OnParticleCollision(other : GameObject) {
	var weapon : Weapon = other.GetComponent("Weapon") as Weapon;
	if(!weapon || weapon.Vulnerability != Vulnerability) return;
	var damage : float = weapon.Damage;
	
	var ps : ParticleSystem = other.particleSystem;
	
	// adjust array length
	var safeLength = ps.safeCollisionEventSize;
	if (collisionEvents.Length < safeLength) {
		collisionEvents = new ParticleSystem.CollisionEvent[safeLength];
	}
	// get collision events for the gameObject that the script is attached to
	var numCollisionEvents = ps.GetCollisionEvents(gameObject, collisionEvents);
	
	if(damage > 0) {
		// apply some force to RigidBody components
		for (var i = 0; i < numCollisionEvents; i++) {
			if (gameObject.rigidbody)
			{
				var pos = collisionEvents[i].intersection;
				var force = collisionEvents[i].velocity * Punch;
				gameObject.rigidbody.AddForce(force,ForceMode.Impulse);
			}
		}
			
		var drops = Mathf.Round(numCollisionEvents * BloodPerHit);
		gameObject.particleSystem.Emit(drops);
			
		if(fxDeadline < Time.time && gameObject.transform.localScale.y > Size.y / 2) {
			fxDeadline = Time.time + fxCoolDown;
			var painLevel = Mathf.Min(Mathf.Floor(drops / 50 * Pain.length), Pain.length-1);
			gameObject.audio.PlayOneShot(Pain[painLevel]);
		}
	}
	
	gameObject.transform.localScale /= 1 + Mathf.Log(1 + Mathf.Min(numCollisionEvents * damage / Armor, 1.0) * 1000) / Mathf.Log(10) / 3;
	
	if(gameObject.transform.localScale.y < Size.y * Destruction) {
		Destroy(gameObject);
	}
	

}

